==> payara-embedded-all-5.193.1-sources.jar: fish/payara/microprofile/faulttolerance/policy/CircuitBreakerPolicy.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2019 Payara Foundation and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://github.com/payara/Payara/blob/master/LICENSE.txt
 * See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * The Payara Foundation designates this particular file as subject to the "Classpath"
 * exception as provided by the Payara Foundation in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package fish.payara.microprofile.faulttolerance.policy;

import java.lang.reflect.Method;
import java.time.temporal.ChronoUnit;
import java.util.logging.Logger;

import javax.interceptor.InvocationContext;

import org.eclipse.microprofile.faulttolerance.CircuitBreaker;

import fish.payara.microprofile.faulttolerance.FaultToleranceConfig;

/**
 * The resolved "cached" information of a {@link CircuitBreaker} annotation an a specific method.
 * 
 * @author Jan Bernitt
 */
public final class CircuitBreakerPolicy extends Policy {

    static final Logger logger = Logger.getLogger(CircuitBreakerPolicy.class.getName());

    public final Class<? extends Throwable>[] failOn;
    public final long delay;
    public final ChronoUnit delayUnit;
    public final int requestVolumeThreshold;
    public final double failureRatio;
    public final int successThreshold;

    public CircuitBreakerPolicy(Method annotatedMethod, Class<? extends Throwable>[] failOn, long delay, ChronoUnit delayUnit,
            int requestVolumeThreshold, double failureRatio, int successThreshold) {
        checkAtLeast(0, annotatedMethod, CircuitBreaker.class, "delay", delay);
        checkAtLeast(1, annotatedMethod, CircuitBreaker.class, "requestVolumeThreshold", requestVolumeThreshold);
        checkAtLeast(0d, annotatedMethod, CircuitBreaker.class, "failureRatio", failureRatio);
        checkAtMost(1.0d, annotatedMethod, CircuitBreaker.class, "failureRatio", failureRatio);
        checkAtLeast(1, annotatedMethod, CircuitBreaker.class, "successThreshold", successThreshold);
        this.failOn = failOn;
        this.delay = delay;
        this.delayUnit = delayUnit;
        this.requestVolumeThreshold = requestVolumeThreshold;
        this.failureRatio = failureRatio;
        this.successThreshold = successThreshold;
    }

    public static CircuitBreakerPolicy create(InvocationContext context, FaultToleranceConfig config) {
        if (config.isAnnotationPresent(CircuitBreaker.class) && config.isEnabled(CircuitBreaker.class)) {
            CircuitBreaker annotation = config.getAnnotation(CircuitBreaker.class);
            return new CircuitBreakerPolicy(context.getMethod(),
                    config.failOn(annotation),
                    config.delay(annotation),
                    config.delayUnit(annotation),
                    config.requestVolumeThreshold(annotation),
                    config.failureRatio(annotation),
                    config.successThreshold(annotation));
        }
        return null;
    }

    /**
     * Helper method that checks whether or not the given exception is included in the failOn parameter.
     * 
     * @param ex The exception to check
     * @return True if the exception is covered by {@link #failOn} list of this policy
     */
    public boolean failOn(Exception ex) {
        return Policy.isCaught(ex, failOn);
    }
}

==> payara-embedded-all-6.2023.12-sources.jar: fish/payara/microprofile/faulttolerance/policy/CircuitBreakerPolicy.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2019-2021 Payara Foundation and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://github.com/payara/Payara/blob/master/LICENSE.txt
 * See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * The Payara Foundation designates this particular file as subject to the "Classpath"
 * exception as provided by the Payara Foundation in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package fish.payara.microprofile.faulttolerance.policy;

import java.lang.reflect.Method;
import java.time.temporal.ChronoUnit;

import jakarta.interceptor.InvocationContext;

import org.eclipse.microprofile.faulttolerance.CircuitBreaker;

import fish.payara.microprofile.faulttolerance.FaultToleranceConfig;

/**
 * The resolved "cached" information of a {@link CircuitBreaker} annotation an a specific method.
 * 
 * @author Jan Bernitt
 */
public final class CircuitBreakerPolicy extends Policy {

    private final Class<? extends Throwable>[] failOn;
    private final Class<? extends Throwable>[] skipOn;
    public final long delay;
    public final ChronoUnit delayUnit;
    public final int requestVolumeThreshold;
    public final double failureRatio;
    public final int successThreshold;

    public CircuitBreakerPolicy(Method annotatedMethod, Class<? extends Throwable>[] failOn,
            Class<? extends Throwable>[] skipOn, long delay, ChronoUnit delayUnit, int requestVolumeThreshold,
            double failureRatio, int successThreshold) {
        checkAtLeast(0, annotatedMethod, CircuitBreaker.class, "delay", delay);
        checkAtLeast(1, annotatedMethod, CircuitBreaker.class, "requestVolumeThreshold", requestVolumeThreshold);
        checkAtLeast(0d, annotatedMethod, CircuitBreaker.class, "failureRatio", failureRatio);
        checkAtMost(1.0d, annotatedMethod, CircuitBreaker.class, "failureRatio", failureRatio);
        checkAtLeast(1, annotatedMethod, CircuitBreaker.class, "successThreshold", successThreshold);
        this.failOn = failOn;
        this.skipOn = skipOn;
        this.delay = delay;
        this.delayUnit = delayUnit;
        this.requestVolumeThreshold = requestVolumeThreshold;
        this.failureRatio = failureRatio;
        this.successThreshold = successThreshold;
    }

    public static CircuitBreakerPolicy create(InvocationContext context, FaultToleranceConfig config) {
        if (config.isAnnotationPresent(CircuitBreaker.class) && config.isEnabled(CircuitBreaker.class)) {
            CircuitBreaker annotation = config.getAnnotation(CircuitBreaker.class);
            return new CircuitBreakerPolicy(context.getMethod(),
                    config.failOn(annotation),
                    config.skipOn(annotation),
                    config.delay(annotation),
                    config.delayUnit(annotation),
                    config.requestVolumeThreshold(annotation),
                    config.failureRatio(annotation),
                    config.successThreshold(annotation));
        }
        return null;
    }

    /**
     * Helper method that checks whether or not the given exception is considered a success or failure.
     * 
     * Relevant part from the {@link CircuitBreaker} documentation:
     * <blockquote>
     * When a method returns a result, the following rules are applied to determine whether the result is a success or a failure:
     * <ul>
     * <li>If the method does not throw a {@link Throwable}, it is considered a success
     * <li>Otherwise, if the thrown object is assignable to any value in the {@link #skipOn()} parameter, is is considered a success
     * <li>Otherwise, if the thrown object is assignable to any value in the {@link #failOn()} parameter, it is considered a failure
     * <li>Otherwise it is considered a success
     * </ul>
     * </blockquote>
     * 
     * @param ex The exception to check
     * @return True if the exception is considered a failure, false if it is considered a success.
     */
    public boolean isFailure(Throwable ex) {
        return !Policy.isCaught(ex, skipOn) && Policy.isCaught(ex, failOn);
    }
}
